#!/bin/bash

state="$(set +o)"
set -eu -o pipefail

exec 2>&1

if [ "${PT__noop:-}" = true ]; then
  return 0
fi

orig_noop="${PT__noop:-}"

test_support() {
  (
    set +vx; eval "$state"
    export PT__noop=true
    source  "$PT__installdir/puppet_agent/tasks/install_shell.sh"
    curl -o /dev/null --silent --fail --head "$download_url"
  )
  return $?
}

# Try to delegate to standard install script first
if [ -f "$PT__installdir/puppet_agent/tasks/install_shell.sh" ]; then
  if test_support; then
    if [ "$orig_noop" = true ]; then
      exit 0
    fi

    echo "Detected standard installation support works, delegating"
    exec bash "$PT__installdir/puppet_agent/tasks/install_shell.sh"
  fi
fi

echo "Standard installation support unavailable, using distro packages"

if [ -f "$PT__installdir/facts/tasks/bash.sh" ]; then
  platform=$(bash $PT__installdir/facts/tasks/bash.sh "platform")
  platform_version=$(bash $PT__installdir/facts/tasks/bash.sh "release")
else
  exit 1
fi

case "$platform" in
Debian|Ubuntu)
  export DEBIAN_FRONTEND=noninteractive
  apt-get update -q

  if [ -n "${PT_version:-}" ]; then
    apt-get install -y "puppet-agent=${PT_version}" ruby-puppet-resource-api
  else
    apt-get install -y puppet-agent
  fi
;;
RedHat|CentOS|Fedora|AlmaLinux|RockyLinux)
  if [ -n "${PT_version:-}" ]; then
    dnf install -y puppet="${PT_version}"
  else
    dnf install -y puppet
  fi
;;
*)
  echo "Error: unsupported platform $platform, sorry!"
  exit 1
;;
esac

# Deal with hardcoded paths in some bolt modules
mkdir -p /opt/puppetlabs/puppet/bin
cd /opt/puppetlabs/puppet/bin

for bin_name in ruby puppet hiera facter gem; do
  if [ -e "./${bin_name}" ]; then
    continue
  fi

  cat > "./${bin_name}" <<'EOF'
#!/bin/bash
# This is a shim for hardcoded puppet paths, to redirect to the corresponding binary
# in PATH
exec "$(basename "$0")" "$@"
EOF
  chmod 755 "./${bin_name}"
done
